#!/bin/bash
#
# Fix pdftex.ini and pdftex.fmt on tetex 3 systems. Ian Hutchinson 2007.  
#
# Recent versions of pdftex have caused a bug in pdftex
# alignment when operating on plain tex files that contain
# \magnification. The authors of pdftex consider the problem to be in
# the plain tex format, and have basically refused to do anything about
# it in pdftex. tetex is now not maintained.  Therefore although the
# bug can be fixed very easily by changing the format, it probably
# won't happen in the distribution. That is why I am releasing this
# script on an unsuspecting world(!) All it does is add some commands
# to pdftex.ini and redump the formats. If you run it a second time, it
# will undo the process and return the buggy format.  
#
# Toggle legalities: No waranty of any kind is offered or implied. If
# you run this script you do so purely at your own risk. Toggle.

PDFTEXINIS=`locate plain/config/pdftex.ini`;
if [ -z "$PDFTEXINIS" ] ; then
   echo pdftex.ini file not located. Searching standard place.
   if [ -z "`/bin/ls /usr/share/texmf/tex/plain/config/pdftex.ini 2>&1 | fgrep 'No such'`" ] ; then
	PDFTEXINIS=`/bin/ls /usr/share/texmf/tex/plain/config/pdftex.ini`;
	echo Found standard file: $PDFTEXINIS
   else
	echo No PDFTEXINI files found.
	exit
   fi
else
    echo Located the following pdftex.ini files: 
    echo $PDFTEXINIS
fi
for PDFTEXINI in $PDFTEXINIS; do
# Only fix files that end in .ini
if [ ${PDFTEXINI%.ini}.ini. == $PDFTEXINI. ] ; then
echo;
echo Examining file: $PDFTEXINI
if [ -f "$PDFTEXINI" ] ; then
    DONE=`grep IHH $PDFTEXINI`
    if [ -z "$DONE" ]; then
	echo You need pdftex.ini to be replaced, and new formats dumped.
	read -p "Would you like me to do that now for you? [y]es [n]o: " YESNO
	if [ $YESNO. = "y." ] ; then
	    if cp $PDFTEXINI $PDFTEXINI.old ; then 
# Note that in the following here document the backquotes have to be 
# backslash-escaped in bash (the escapes disappear), but not of course in tex. 
		cat >$PDFTEXINI <<EOF
% Thomas Esser, 1998, 2004. public domain.
\ifx\pdfoutput\undefined
\else
  \ifx\pdfoutput\relax
  \else
    \input pdftexconfig
    \pdfoutput=1
  \fi
\fi

\input plain
\message{Starting pdftex magnification patch. IHHutchinson, 2007}
\def\IHHpdftexpatched{Yes}
\ifx\pdfoutput\undefined
\else
  \ifx\pdfoutput\relax
  \else
\catcode\`@=11
\def\m@g{%
\mag\count@
\hsize6.5truein\vsize8.9truein\dimen\footins8truein
%These commands fix the pdf sizes to be in true units.
\mag@fix\pdfhorigin \mag@fix\pdfvorigin
\mag@fix\pdfpagewidth \mag@fix\pdfpageheight
}
\def\mag@fix#1{%
 #1=\expandafter\convert@truept\the#1\relax
}
\begingroup
\catcode\`P=12
\catcode\`T=12
\lowercase{%
\endgroup
\def\convert@truept#1PT{%
#1truept%
}%
}%
\catcode\`@=12
  \fi
\fi

\dump
\endinput

EOF
		echo Ok. Saved original file as $PDFTEXINI.old 
		fmtutil-sys --byfmt pdftex
# The fmtutil does essentially this but better:
#		cd /usr/share/texmf-var/web2c
#		pdftex -ini -jobname=pdftex -progname=pdftex -translate-file=cp227.tcx *pdftex.ini
#		cd -
		echo Ran the pdftex ini dump. Formats now should be fixed. 
            else
		echo Failed: could not copy the original pdftex.ini file.
		echo You probably need to run this script as root or ask your sysadmin.
	    fi
	fi
    else
	if [ -f $PDFTEXINI.old ]; then
	    echo Your pdftex.ini was already fixed by this script.
            read -p "Do you want to restore the old one? [y]es [n]o: " YESNO
	    if [ $YESNO. = "y." ]; then
	      if cp $PDFTEXINI.old $PDFTEXINI ; then
		fmtutil-sys --byfmt pdftex 
		echo Ran the pdftex ini dump. Old formats now should be restored.
	      else
		echo Could not copy file. You need to be root.
              fi
	    fi
	else
	    echo Seems to have been fixed by hand.
	fi
    fi
else
    echo I can''t seem find pdftex.ini. The locate database may not be up to date.
fi
fi
done
echo Whatever you have done can be reversed by running this script again.
echo Individual users may have to regenerate their formats by 
echo fmtutil --byfmt pdftex 
